/*globals require, window*/

require([
    'jquery',
    'ep',
    'testsuite/qunit',
    'de_epages/molliepayments/ui/mbosettings',
    'testsuite/qunit-fixture!de_epages/molliepayments/ui/mbosettings-test',

    'jquery/metaparse'
], function ($, ep, QUnit, mboSettings, fixture) {
    'use strict';

    var testAjaxResponse = function(objectId, profileId) {
        // test data
        QUnit.equal(
            objectId.test,
            objectId.orig,
            "AJAX request (change profile) - correct objectId '" + objectId.orig + "''"
        );
        QUnit.equal(
            profileId.test,
            profileId.orig,
            "AJAX request (change profile) - Correct profile ID '" + profileId.orig + "''"
        );
    }

    // mock ajax object
    ep.ajax = function (o) {
        var result = $.Deferred();
        if (o.data && o.data.ChangeAction) {
            switch(o.data.ChangeAction) {
                case 'GetMolliePaymentsMethods':
                    // unlimited payment methods
                    if (o.data.ObjectID === 99) {

                        if (o.data.IsInTestMode === "1") {
                            // test data
                            testAjaxResponse({
                                test: o.data.ObjectID,
                                orig: objectId
                            }, {
                                test: o.data.ProfileId,
                                orig: profilesList.profiles[0].id
                            });

                            // resolve promise
                            result.resolveWith(result, [methodsResponse]);
                        } else if (o.data.IsInTestMode === "0") {
                            // test data
                            testAjaxResponse({
                                test: o.data.ObjectID,
                                orig: objectId
                            }, {
                                test: o.data.ProfileId,
                                orig: profilesList.profiles[0].id
                            });

                            // resolve promise
                            result.resolveWith(result, [{methods: []}]);
                        }

                    // limited payment methods
                    } else if (o.data.ObjectID === 100) {
                        if (o.data.IsInTestMode === "1") {
                            // test data
                            testAjaxResponse({
                                test: o.data.ObjectID,
                                orig: objectIdLimited
                            }, {
                                test: o.data.ProfileId,
                                orig: profilesList.profiles[0].id
                            });

                            // resolve promise
                            result.resolveWith(result, [methodsResponseLimited]);
                        }

                    }
                    break;
                default:
                    result.resolveWith(result, [{
                        "success": false,
                        "error": {
                            msg: "unknownError"
                        }
                    }]);

            }

        }
        return result;
    };

    var profilesList = {"previous":null,"next":null,"profiles":[{"website":"https://www.epages.com","ItemNo":0,"name":"ePages Test Merchant Website 11181230","id":"pfl_wAcmbPdHUs"},{"website":"https://epages.com","ItemNo":1,"name":"ePages GmbH","id":"pfl_tQwb4H8jHz"}]},
        methodsResponse = {
           "methods" : [
              {
                 "id" : "creditcard",
                 "description" : "Kreditkarte"
              },
              {
                 "id" : "belfius",
                 "description" : "Belfius Pay Button"
              },
              {
                 "id" : "giropay",
                 "description" : "giropay"
              },
              {
                 "id" : "kbc",
                 "description" : "KBC/CBC Zahlungsbutton"
              },
              {
                 "id" : "ideal",
                 "description" : "iDEAL"
              },
              {
                 "id" : "sofort",
                 "description" : "SOFORT Überweisung"
              },
              {
                 "id" : "banktransfer",
                 "description" : "Überweisung"
              },
              {
                 "id" : "bancontact",
                 "description" : "Bancontact"
              },
              {
                 "id" : "eps",
                 "description" : "eps"
              },
              {
                 "id" : "paypal",
                 "description" : "PayPal"
              },
              {
                 "id" : "przelewy24",
                 "description" : "Przelewy24"
              },
              {
                 "id" : "voucher",
                 "description" : "Vouchers"
              },
              {
                 "id" : "klarnapaylater",
                 "description" : "Rechnung."
              },
              {
                 "id" : "klarnasliceit",
                 "description" : "Ratenkauf."
              },
              {
                 "id" : "klarnapaynow",
                 "description" : "Pay now."
              }
           ]
        },
        methodsResponseLimited = {
           "methods" : [
              {
                 "id" : "creditcard",
                 "description" : "Kreditkarte"
              }
           ]
        },
        objectId = 99,
        objectIdLimited = 100;

    QUnit.module('de_epages/molliepayments/ui/mbosettings', {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.asyncTest("Mollie MBO Settings Module Prototype", function () {
        // initialize mbo settings object with default values
        var mboSettingsObj = mboSettings({});

        QUnit.equal(
            $.isFunction(mboSettings),
            true,
            '$.isFunction(mboSettings):'
        );

        $.each([
            'onChangeProfileId',
            'onChangeTestMode',
            'onClickGetProfileList'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction(mboSettingsObj[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest('Test UI functionality', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        // metaparse form elements
        $('#qunit-fixture .ep-js').metaparse();

        var mollieMethod = document.querySelector("#" + "mollieMethodId"),
            mboSettingsObj;

        // instantiate settings object
        mboSettingsObj = mboSettings({
            profilesList: profilesList,
            errorMessages: {
              unknownError: "{UnityUnknownError}",
              connectionError: "{ConnectionErrorRetryLater}",
            },
            objectId: objectId,
            profileSelect: document.getElementById("ProfileId"),
            methodSelect: document.getElementById("Method"),
            testModeSelect: document.getElementById("IsInTestMode"),
            previousMollieProfiles: document.getElementById("PreviousMollieProfiles"),
            nextMollieProfiles: document.getElementById("NextMollieProfiles"),
            hiddenMethodIdInput: mollieMethod
        });

        // select first profile
        mboSettingsObj.profileSelect.selectedIndex = 1;
        mboSettingsObj.profileSelect.dispatchEvent(new Event("change"));

        // wait for mocked ajax response
        window.setTimeout(function() {
            QUnit.equal(
                mboSettingsObj.methodSelect.length,
                methodsResponse.methods.length + 1,
                'DropDown to select payment methods contains ' + (methodsResponse.methods.length + 1) + ' entries (test mode).'
            );
            QUnit.equal(
                mboSettingsObj.methodSelect.options[1].value,
                methodsResponse.methods[0].id,
                'First entry of DropDown to select payment methods contains the correct id "' + methodsResponse.methods[0].id + '".'
            );

            console.log('methodsSelect: ', mboSettingsObj.methodSelect.length);

            // select live mode
            mboSettingsObj.testModeSelect.selectedIndex = 0;
            mboSettingsObj.testModeSelect.dispatchEvent(new Event("change"));

            window.setTimeout(function() {
                QUnit.equal(
                    mboSettingsObj.methodSelect.disabled,
                    true,
                    'DropDown to select payment methods is disabled (no payment methods for live mode).'
                );

                // select test mode again
                mboSettingsObj.testModeSelect.selectedIndex = 1;
                mboSettingsObj.testModeSelect.dispatchEvent(new Event("change"));

                window.setTimeout(function() {
                    QUnit.equal(
                        mboSettingsObj.methodSelect.length,
                        methodsResponse.methods.length + 1,
                        'DropDown to select payment methods contains ' + (methodsResponse.methods.length + 1) + ' entries again (test mode).'
                    );

                    // select first entry => payment methods select becomes disabled
                    mboSettingsObj.profileSelect.selectedIndex = 0;
                    mboSettingsObj.profileSelect.dispatchEvent(new Event("change"));

                    QUnit.equal(
                        mboSettingsObj.methodSelect.disabled,
                        true,
                        'DropDown to select payment methods is disabled.'
                    );

                    QUnit.start();
                }, 50);
            }, 50);

        }, 50);
    });

    QUnit.asyncTest('test behaviour for limited feature', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        // metaparse form elements
        $('#qunit-fixture .ep-js').metaparse();

        var mollieMethod = document.querySelector("#" + "mollieMethodId"),
            mboSettingsObj;

        // instantiate settings object
        mboSettingsObj = mboSettings({
            profilesList: profilesList,
            errorMessages: {
              unknownError: "{UnityUnknownError}",
              connectionError: "{ConnectionErrorRetryLater}",
            },
            objectId: objectIdLimited,
            profileSelect: document.getElementById("ProfileId"),
            methodSelect: document.getElementById("Method"),
            testModeSelect: document.getElementById("IsInTestMode"),
            previousMollieProfiles: document.getElementById("PreviousMollieProfiles"),
            nextMollieProfiles: document.getElementById("NextMollieProfiles"),
            hiddenMethodIdInput: mollieMethod
        });

        // select first profile
        mboSettingsObj.profileSelect.selectedIndex = 1;
        // $(mboSettingsObj.profileSelect).trigger('change');
        mboSettingsObj.profileSelect.dispatchEvent(new Event("change"));

        // wait for mocked ajax response
        window.setTimeout(function() {
            QUnit.equal(
                mboSettingsObj.methodSelect.length,
                methodsResponseLimited.methods.length,
                'DropDown to select payment methods contains ' + (methodsResponseLimited.methods.length) + ' entry exactly.'
            );
            QUnit.equal(
                mboSettingsObj.methodSelect.options[0].value,
                methodsResponseLimited.methods[0].id,
                'Only entry of DropDown to select payment methods contains the correct id "' + methodsResponseLimited.methods[0].id + '".'
            );

            QUnit.equal(
                mboSettingsObj.methodSelect.disabled,
                true,
                'DropDown to select payment methods is disabled (limited to one payment method).'
            );

            // select "select entry" option in profile dropdown
            mboSettingsObj.profileSelect.selectedIndex = 0;
            mboSettingsObj.profileSelect.dispatchEvent(new Event("change"));

            window.setTimeout(function() {
                QUnit.equal(
                    mboSettingsObj.methodSelect.disabled,
                    true,
                    'DropDown to select payment methods is disabled.'
                );
                QUnit.equal(
                    mboSettingsObj.methodSelect.options.length,
                    0,
                    'DropDown to select payment methods contains no entries.'
                );

                QUnit.start();
            }, 50);

        }, 50);
    });

    QUnit.asyncTest('test public event handler', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        // metaparse form elements
        $('#qunit-fixture .ep-js').metaparse();

        var mollieMethod = document.querySelector("#" + "mollieMethodId"),
            mboSettingsObj;

        // instantiate settings object
        mboSettingsObj = mboSettings({
            profilesList: profilesList,
            errorMessages: {
              unknownError: "{UnityUnknownError}",
              connectionError: "{ConnectionErrorRetryLater}",
            },
            objectId: objectId,
            profileSelect: document.getElementById("ProfileIdOneProfile"),
            methodSelect: document.getElementById("Method"),
            testModeSelect: document.getElementById("IsInTestMode"),
            previousMollieProfiles: document.getElementById("PreviousMollieProfiles"),
            nextMollieProfiles: document.getElementById("NextMollieProfiles"),
            hiddenMethodIdInput: mollieMethod
        });

        // test handler fired if profile changed
        mboSettingsObj.onChangeProfileId(document.getElementById("ProfileIdOneProfile"));

        // wait for mocked ajax response
        window.setTimeout(function() {
            QUnit.equal(
                mboSettingsObj.methodSelect.length,
                methodsResponse.methods.length + 1,
                'DropDown to select payment methods contains ' + (methodsResponse.methods.length + 1) + ' entries (test mode).'
            );
            QUnit.equal(
                mboSettingsObj.methodSelect.options[1].value,
                methodsResponse.methods[0].id,
                'First entry of DropDown to select payment methods contains the correct id "' + methodsResponse.methods[0].id + '".'
            );
            QUnit.equal(
                mboSettingsObj.methodSelect.options[1].value === methodsResponse.methods[0].id && mboSettingsObj.methodSelect.length === (methodsResponse.methods.length + 1),
                true,
                'Public event handler which is fired if profiles change works correct.'
            );

            // select live mode and execute corresponding event handler programatically
            mboSettingsObj.testModeSelect.selectedIndex = 0;
            mboSettingsObj.onChangeTestMode(document.getElementById("IsInTestMode"));

            window.setTimeout(function() {
                QUnit.equal(
                    mboSettingsObj.methodSelect.disabled,
                    true,
                    'DropDown to select payment methods is disabled.'
                );
                QUnit.equal(
                    mboSettingsObj.methodSelect.options.length,
                    0,
                    'DropDown to select payment methods contains no entries.'
                );

                QUnit.start();
            }, 50);

        }, 50);
    });
});