/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/remotesearch/ui/searchfilter-slider-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/remotesearch/ui/searchfilter-slider',
    "$dict!de_epages/remotesearch/dictionary",

    '$ready!'
], function (QUnit, fixture, sinon, $, searchFilterSlider, remotesearchDict) {

    QUnit.module("de_epages/remotesearch/ui/searchfilter-slider", {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.test('de_epages/remotesearch/ui/searchfilter-slider - check existence of search filter slider methods', 9, function () {
        $.each([
            '_create',
            '_resize',
            '_buildUI',
            '_initSlider',
            '_buildScale',
            '_setOption',
            '_setNewValue',
            '_createLinearGradient',
            'destroy'
        ], function () {
            QUnit.ok(
                $.isFunction($.ui.remotesearchUiSearchFilterSlider.prototype[this]),
                this + ' is a method of the searchfilter-slider widget'
            );
        });
    });

    QUnit.asyncTest("de_epages/remotesearch/ui/searchfilter-slider - UI tests for search filter slider", function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        var $searchFilterSlider = $('.ep-searchfilter-facetsearch-startdepth'),
            steps = 4,
            startValue = 2,
            minSelectable = 1,
            newValue = 0,
            searchFilterWidget,
            sliderWidget,
            resizeSpy;

        // instantiate videodialog widget
        $searchFilterSlider.remotesearchUiSearchFilterSlider({
            steps: steps,
            value: startValue,
            minSelectable: 0
        });

        // wait until widget is instantiated
        window.setTimeout(function() {
            QUnit.equal(
                !!$searchFilterSlider.remotesearchUiSearchFilterSlider('instance'),
                true,
                'remotesearchUiSearchFilterSlider widget instance exists.'
            );

            searchFilterWidget = $searchFilterSlider.remotesearchUiSearchFilterSlider('instance');
            sliderWidget = searchFilterWidget.slider.slider('instance');

            // register spy to investigate hide method
            resizeSpy = sinon.spy(searchFilterWidget, '_buildScale');

            QUnit.equal(
                searchFilterWidget.dom.scaleList.find('li').length,
                steps + 1,
                'Slider scale contains ' + (steps + 1) + ' labels.'
            );

            QUnit.equal(
                searchFilterWidget.dom.scaleList.find('li').length - 1,
                steps,
                'Slider contains ' + steps + ' possible steps.'
            );

            QUnit.equal(
                searchFilterWidget.dom.facetedSearchStartDepth.length,
                1,
                'searchFilterSlider template contains "1" input field to represent the selected value.'
            );

            QUnit.equal(
                Number(searchFilterWidget.dom.facetedSearchStartDepth.val()),
                startValue,
                'Start value "' + startValue + '" was set correctly as value to input element'
            );


            QUnit.equal(
                searchFilterWidget.dom.infoContent.text(),
                remotesearchDict.translate("SearchFilterStartDepthFromSub"),
                'Information text is set correctly to "' + remotesearchDict.translate("SearchFilterStartDepthFromSub") + '".'
            );

            $searchFilterSlider.remotesearchUiSearchFilterSlider('option', 'value', newValue);

            QUnit.equal(
                Number(searchFilterWidget.dom.facetedSearchStartDepth.val()),
                newValue,
                'New value "' + newValue + '" was set correctly as value to input element'
            );

            QUnit.equal(
                searchFilterWidget.dom.infoContent.text(),
                remotesearchDict.translate("SearchFilterStartDepthFromSub"),
                'Information text was changed correctly to "' + remotesearchDict.translate("SearchFilterStartDepthFromSub") + '".'
            );

            // set new minimal selectable step
            $searchFilterSlider.remotesearchUiSearchFilterSlider('option', 'minSelectable', minSelectable);

            QUnit.equal(
                Number(searchFilterWidget.dom.facetedSearchStartDepth.val()),
                minSelectable,
                'Current value "' + newValue + '" is not selectable anymore and was corrected to the min. value "' + minSelectable + '".'
            );


            // trigger resize event
            $(window).trigger('resize.remotesearchUiSearchFilterSlider');

            // wait for resize bouncer
            window.setTimeout(function() {
                QUnit.ok(
                    resizeSpy.callCount >= 1,
                    'Method to resize scale was called ' + resizeSpy.callCount + ' times.'
                );

                // destroy widget
                $searchFilterSlider.remotesearchUiSearchFilterSlider('destroy');
                // wait until widget is destroyed
                window.setTimeout(function() {
                    QUnit.equal(
                        $searchFilterSlider.remotesearchUiSearchFilterSlider('instance') === undefined,
                        true,
                        'Widget was destroyed.'
                    );
                    QUnit.equal(
                        $searchFilterSlider.children().length,
                        0,
                        'Element content was removed from DOM.'
                    );

                    QUnit.start();
                }, 500);
            }, 150);
        }, 500);
    });
});